@@ -4,7 +4,7 @@
 # concepts of ports
 
 use strict;
-use lib '..';
+use lib '../lib';
 use GraphViz;
 
 my $g = GraphViz->new();
